@@ -3,9 +3,17 @@
 See the docstring for pymc3.backends for more information (including
 creating custom backends).
 """
+import itertools as itl
+import logging
+
 import numpy as np
-from ..model import modelcontext
 import warnings
+import theano.tensor as tt
+
+from ..model import modelcontext
+from .report import SamplerReport, merge_reports
+
+logger = logging.getLogger('pymc3')
 
 
 class BackendError(Exception):
@@ -24,11 +32,13 @@ class BaseTrace(object):
     vars : list of variables
         Sampling values will be stored for these variables. If None,
         `model.unobserved_RVs` is used.
+    test_point : dict
+        use different test point that might be with changed variables shapes
     """
 
     supports_sampler_stats = False
 
-    def __init__(self, name, model=None, vars=None):
+    def __init__(self, name, model=None, vars=None, test_point=None):
         self.name = name
 
         model = modelcontext(model)
@@ -41,7 +51,13 @@ class BaseTrace(object):
 
         # Get variable shapes. Most backends will need this
         # information.
-        var_values = list(zip(self.varnames, self.fn(model.test_point)))
+        if test_point is None:
+            test_point = model.test_point
+        else:
+            test_point_ = model.test_point.copy()
+            test_point_.update(test_point)
+            test_point = test_point_
+        var_values = list(zip(self.varnames, self.fn(test_point)))
         self.var_shapes = {var: value.shape
                            for var, value in var_values}
         self.var_dtypes = {var: value.dtype
@@ -49,6 +65,10 @@ class BaseTrace(object):
         self.chain = None
         self._is_base_setup = False
         self.sampler_vars = None
+        self._warnings = []
+
+    def _add_warnings(self, warnings):
+        self._warnings.extend(warnings)
 
     # Sampling methods
 
@@ -72,7 +92,6 @@ class BaseTrace(object):
 
         self.sampler_vars = sampler_vars
 
-
     def setup(self, draws, chain, sampler_vars=None):
         """Perform chain-specific setup.
 
@@ -163,7 +182,7 @@ class BaseTrace(object):
             return self._get_sampler_stats(varname, sampler_idx, burn, thin)
 
         sampler_idxs = [i for i, s in enumerate(self.sampler_vars)
-                       if varname in s]
+                        if varname in s]
         if not sampler_idxs:
             raise KeyError("Unknown sampler stat %s" % varname)
 
@@ -174,20 +193,19 @@ class BaseTrace(object):
         else:
             return vals
 
-
     def _get_sampler_stats(self, varname, sampler_idx, burn, thin):
         """Get sampler statistics."""
         raise NotImplementedError()
 
     def _slice(self, idx):
         """Slice trace object."""
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def point(self, idx):
         """Return dictionary of point values at `idx` for current chain
         with variables names as keys.
         """
-        raise NotImplementedError
+        raise NotImplementedError()
 
     @property
     def stat_names(self):
@@ -247,6 +265,11 @@ class MultiTrace(object):
                 raise ValueError("Chains are not unique.")
             self._straces[strace.chain] = strace
 
+        self._report = SamplerReport()
+        for strace in straces:
+            if hasattr(strace, '_warnings'):
+                self._report._add_warnings(strace._warnings, strace.chain)
+
     def __repr__(self):
         template = '<{}: {} chains, {} iterations, {} variables>'
         return template.format(self.__class__.__name__,
@@ -260,6 +283,10 @@ class MultiTrace(object):
     def chains(self):
         return list(sorted(self._straces.keys()))
 
+    @property
+    def report(self):
+        return self._report
+
     def __getitem__(self, idx):
         if isinstance(idx, slice):
             return self._slice(idx)
@@ -292,7 +319,7 @@ class MultiTrace(object):
         raise KeyError("Unknown variable %s" % var)
 
     _attrs = set(['_straces', 'varnames', 'chains', 'stat_names',
-                  'supports_sampler_stats'])
+                  'supports_sampler_stats', '_report'])
 
     def __getattr__(self, name):
         # Avoid infinite recursion when called before __init__
@@ -336,6 +363,39 @@ class MultiTrace(object):
                 names.update(vars.keys())
         return names
 
+    def add_values(self, vals):
+        """add values to traces.
+        Parameters
+        ----------
+        vals : dict (str: array-like)
+             The keys should be the names of the new variables. The values are
+             expected to be array-like object.
+             For traces with more than one chain the lenght of each value
+             should match the number of total samples already in the trace
+             (chains * iterations), otherwise a warning is raised.
+        """
+        for k, v in vals.items():
+            if k in self.varnames:
+                raise ValueError("Variable name {} already exists.".format(k))
+
+            self.varnames.append(k)
+
+            chains = self._straces
+            l_samples = len(self) * len(self.chains)
+            l_v = len(v)
+            if l_v != l_samples:
+                warnings.warn("The lenght of the values you are trying to "
+                              "add ({}) does not match the number ({}) of "
+                              "total samples in the trace "
+                              "(chains * iterations)".format(l_v, l_samples))
+
+            v = np.squeeze(v.reshape(len(chains), len(self), -1))
+
+            for idx, chain in enumerate(chains.values()):
+                chain.samples[k] = v[idx]
+                dummy = tt.as_tensor_variable([], k)
+                chain.vars.append(dummy)
+
     def get_values(self, varname, burn=0, thin=1, combine=True, chains=None,
                    squeeze=True):
         """Get values from traces.
@@ -403,10 +463,13 @@ class MultiTrace(object):
                    for chain in chains]
         return _squeeze_cat(results, combine, squeeze)
 
-    def _slice(self, idx):
-        """Return a new MultiTrace object sliced according to `idx`."""
-        new_traces = [trace._slice(idx) for trace in self._straces.values()]
-        return MultiTrace(new_traces)
+    def _slice(self, slice):
+        """Return a new MultiTrace object sliced according to `slice`."""
+        new_traces = [trace._slice(slice) for trace in self._straces.values()]
+        trace = MultiTrace(new_traces)
+        idxs = slice.indices(len(self))
+        trace._report = self._report._slice(*idxs)
+        return trace
 
     def point(self, idx, chain=None):
         """Return a dictionary of point values at `idx`.
@@ -421,6 +484,20 @@ class MultiTrace(object):
             chain = self.chains[-1]
         return self._straces[chain].point(idx)
 
+    def points(self, chains=None):
+        """Return an iterator over all or some of the sample points
+
+        Parameters
+        ----------
+        chains : list of int or N
+            The chains whose points should be inlcuded in the iterator.  If
+            chains is not given, include points from all chains.
+        """
+        if chains is None:
+            chains = self.chains
+
+        return itl.chain.from_iterable(self._straces[chain] for chain in chains)
+
 
 def merge_traces(mtraces):
     """Merge MultiTrace objects.
@@ -444,6 +521,7 @@ def merge_traces(mtraces):
             if new_chain in base_mtrace._straces:
                 raise ValueError("Chains are not unique.")
             base_mtrace._straces[new_chain] = strace
+    base_mtrace.report = merge_reports([trace.report for trace in mtraces])
     return base_mtrace
 
 
