@@ -1,3 +1,4 @@
+import theano
 import theano.tensor as tt
 
 from ..model import FreeRV
@@ -7,8 +8,8 @@ from ..math import logit, invlogit
 from .distribution import draw_values
 import numpy as np
 
-__all__ = ['transform', 'stick_breaking', 'logodds', 'interval',
-          'lowerbound', 'upperbound', 'log', 'sum_to_1', 't_stick_breaking']
+__all__ = ['transform', 'stick_breaking', 'logodds', 'interval', 'log_exp_m1',
+           'lowerbound', 'upperbound', 'log', 'sum_to_1', 't_stick_breaking']
 
 
 class Transform(object):
@@ -33,6 +34,7 @@ class Transform(object):
         raise NotImplementedError
 
     def apply(self, dist):
+        # avoid circular import
         return TransformedDistribution.dist(dist, self)
 
     def __str__(self):
@@ -79,6 +81,9 @@ class TransformedDistribution(distribution.Distribution):
         return (self.dist.logp(self.transform_used.backward(x)) +
                 self.transform_used.jacobian_det(x))
 
+    def logp_nojac(self, x):
+        return self.dist.logp(self.transform_used.backward(x))
+
 transform = Transform
 
 
@@ -90,7 +95,7 @@ class Log(ElemwiseTransform):
 
     def forward(self, x):
         return tt.log(x)
-    
+
     def forward_val(self, x, point=None):
         return self.forward(x)
 
@@ -100,18 +105,37 @@ class Log(ElemwiseTransform):
 log = Log()
 
 
+class LogExpM1(ElemwiseTransform):
+    name = "log_exp_m1"
+    
+    def backward(self, x):
+        return tt.nnet.softplus(x)
+    
+    def forward(self, x):
+        """Inverse operation of softplus
+        y = Log(Exp(x) - 1) 
+          = Log(1 - Exp(-x)) + x
+        """
+        return tt.log(1.-tt.exp(-x)) + x
+    
+    def forward_val(self, x, point=None):
+        return self.forward(x)
+    
+    def jacobian_det(self, x):
+        return -tt.nnet.softplus(-x)
+
+log_exp_m1 = LogExpM1()
+
+
 class LogOdds(ElemwiseTransform):
     name = "logodds"
 
-    def __init__(self):
-        pass
-
     def backward(self, x):
         return invlogit(x, 0.0)
 
     def forward(self, x):
         return logit(x)
-    
+
     def forward_val(self, x, point=None):
         return self.forward(x)
 
@@ -245,7 +269,7 @@ class StickBreaking(Transform):
 
     name = "stickbreaking"
 
-    def __init__(self, eps=0.0):
+    def __init__(self, eps=floatX(np.finfo(theano.config.floatX).eps)):
         self.eps = eps
 
     def forward(self, x_):
@@ -258,7 +282,7 @@ class StickBreaking(Transform):
         k = tt.arange(Km1)[(slice(None), ) + (None, ) * (x.ndim - 1)]
         eq_share = logit(1. / (Km1 + 1 - k).astype(str(x_.dtype)))
         y = logit(z) - eq_share
-        return y.T
+        return floatX(y.T)
 
     def forward_val(self, x, point=None):
         return self.forward(x)
@@ -273,7 +297,7 @@ class StickBreaking(Transform):
         yu = tt.concatenate([tt.ones(y[:1].shape), 1 - z])
         S = tt.extra_ops.cumprod(yu, 0)
         x = S * yl
-        return x.T
+        return floatX(x.T)
 
     def jacobian_det(self, y_):
         y = y_.T
@@ -324,6 +348,6 @@ class CholeskyCovPacked(Transform):
 
     def forward_val(self, x, point=None):
         return self.forward(x)
-        
+
     def jacobian_det(self, y):
         return tt.sum(y[self.diag_idxs])
