@@ -2,9 +2,10 @@ import numpy as np
 import numpy.random as nr
 import theano
 import scipy.linalg
+import warnings
 
 from ..distributions import draw_values
-from .arraystep import ArrayStepShared, ArrayStep, metrop_select, Competence
+from .arraystep import ArrayStepShared, PopulationArrayStepShared, ArrayStep, metrop_select, Competence
 import pymc3 as pm
 from pymc3.theanof import floatX
 
@@ -25,6 +26,11 @@ class NormalProposal(Proposal):
         return nr.normal(scale=self.s)
 
 
+class UniformProposal(Proposal):
+    def __call__(self):
+        return nr.uniform(low=-self.s, high=self.s, size=len(self.s))
+
+
 class CauchyProposal(Proposal):
     def __call__(self):
         return nr.standard_cauchy(size=np.size(self.s)) * self.s
@@ -112,7 +118,7 @@ class Metropolis(ArrayStepShared):
         else:
             raise ValueError("Invalid rank for variance: %s" % S.ndim)
 
-        self.scaling = np.atleast_1d(scaling)
+        self.scaling = np.atleast_1d(scaling).astype('d')
         self.tune = tune
         self.tune_interval = tune_interval
         self.steps_until_tune = tune_interval
@@ -167,10 +173,8 @@ class Metropolis(ArrayStepShared):
         return q_new, [stats]
 
     @staticmethod
-    def competence(var):
-        if var.dtype in pm.discrete_types:
-            return Competence.COMPATIBLE
-        return Competence.INCOMPATIBLE
+    def competence(var, has_grad):
+        return Competence.COMPATIBLE
 
 
 def tune(scale, acc_rate):
@@ -348,6 +352,7 @@ class BinaryGibbsMetropolis(ArrayStep):
             return Competence.IDEAL
         return Competence.INCOMPATIBLE
 
+
 class CategoricalGibbsMetropolis(ArrayStep):
     """A Metropolis-within-Gibbs step method optimized for categorical variables.
        This step method works for Bernoulli variables as well, but it is not
@@ -465,16 +470,135 @@ class CategoricalGibbsMetropolis(ArrayStep):
             return Competence.COMPATIBLE
         return Competence.INCOMPATIBLE
 
+
+class DEMetropolis(PopulationArrayStepShared):
+    """
+    Differential Evolution Metropolis sampling step.
+
+    Parameters
+    ----------
+    lamb : float
+        Lambda parameter of the DE proposal mechanism. Defaults to 2.38 / sqrt(2 * ndim)
+    vars : list
+        List of variables for sampler
+    S : standard deviation or covariance matrix
+        Some measure of variance to parameterize proposal distribution
+    proposal_dist : function
+        Function that returns zero-mean deviates when parameterized with
+        S (and n). Defaults to Uniform(-S,+S).
+    scaling : scalar or array
+        Initial scale factor for epsilon. Defaults to 0.001
+    tune : bool
+        Flag for tuning the scaling. Defaults to True.
+    tune_interval : int
+        The frequency of tuning. Defaults to 100 iterations.
+    model : PyMC Model
+        Optional model for sampling step. Defaults to None (taken from context).
+    mode :  string or `Mode` instance.
+        compilation mode passed to Theano functions
+
+    References
+    ----------
+    .. [Braak2006] Cajo C.F. ter Braak (2006).
+        A Markov Chain Monte Carlo version of the genetic algorithm
+        Differential Evolution: easy Bayesian computing for real parameter spaces.
+        Statistics and Computing
+        `link <https://doi.org/10.1007/s11222-006-8769-1>`__
+    """
+    name = 'DEMetropolis'
+
+    default_blocked = True
+    generates_stats = True
+    stats_dtypes = [{
+        'accept': np.float64,
+        'tune': np.bool,
+    }]
+
+    def __init__(self, vars=None, S=None, proposal_dist=None, lamb=None, scaling=0.001,
+                 tune=True, tune_interval=100, model=None, mode=None, **kwargs):
+        warnings.warn('Population based sampling methods such as DEMetropolis are experimental.' \
+            ' Use carefully and be extra critical about their results!')
+
+        model = pm.modelcontext(model)
+
+        if vars is None:
+            vars = model.cont_vars
+        vars = pm.inputvars(vars)
+
+        if S is None:
+            S = np.ones(sum(v.dsize for v in vars))
+
+        if proposal_dist is not None:
+            self.proposal_dist = proposal_dist(S)
+        else:
+            self.proposal_dist = UniformProposal(S)
+
+        self.scaling = np.atleast_1d(scaling).astype('d')
+        if lamb is None:
+            lamb = 2.38 / np.sqrt(2 * S.size)
+        self.lamb = float(lamb)
+        self.tune = tune
+        self.tune_interval = tune_interval
+        self.steps_until_tune = tune_interval
+        self.accepted = 0
+
+        self.mode = mode
+
+        shared = pm.make_shared_replacements(vars, model)
+        self.delta_logp = delta_logp(model.logpt, vars, shared)
+        super(DEMetropolis, self).__init__(vars, shared)
+
+    def astep(self, q0):
+        if not self.steps_until_tune and self.tune:
+            # Tune scaling parameter
+            self.scaling = tune(
+                self.scaling, self.accepted / float(self.tune_interval))
+            # Reset counter
+            self.steps_until_tune = self.tune_interval
+            self.accepted = 0
+
+        epsilon = self.proposal_dist() * self.scaling
+
+        # differential evolution proposal
+        # select two other chains
+        ir1, ir2 = np.random.choice(self.other_chains, 2, replace=False)
+        r1 = self.bij.map(self.population[ir1])
+        r2 = self.bij.map(self.population[ir2])
+        # propose a jump
+        q = floatX(q0 + self.lamb * (r1 - r2) + epsilon)
+
+        accept = self.delta_logp(q, q0)
+        q_new, accepted = metrop_select(accept, q, q0)
+        self.accepted += accepted
+
+        self.steps_until_tune -= 1
+
+        stats = {
+            'tune': self.tune,
+            'accept': np.exp(accept),
+        }
+
+        return q_new, [stats]
+
+    @staticmethod
+    def competence(var, has_grad):
+        if var.dtype in pm.discrete_types:
+            return Competence.INCOMPATIBLE
+        return Competence.COMPATIBLE
+
+
 def sample_except(limit, excluded):
     candidate = nr.choice(limit - 1)
     if candidate >= excluded:
         candidate += 1
     return candidate
 
+
 def softmax(x):
     e_x = np.exp(x - np.max(x))
     return e_x / np.sum(e_x, axis = 0)
 
+
 def delta_logp(logp, vars, shared):
     [logp0], inarray0 = pm.join_nonshared_inputs([logp], vars, shared)
 
