@@ -25,36 +25,16 @@ from ..vartypes import discrete_types
 from ..theanof import inputvars, make_shared_replacements, join_nonshared_inputs
 import numpy.random as nr
 
+from .metropolis import MultivariateNormalProposal
 from .arraystep import metrop_select
 from ..backends import smc_text as atext
 
-__all__ = ['SMC', 'ATMIP_sample']
+__all__ = ['SMC', 'sample_smc']
 
-EXPERIMENTAL_WARNING = "Warning: SMC is an experimental step method, and not yet"\
-    " recommended for use in PyMC3!"
+EXPERIMENTAL_WARNING = ("Warning: SMC is an experimental step method, and not yet "
+                       "recommended for use in PyMC3!")
 
-
-class Proposal(object):
-    """Proposal distributions modified from pymc3 to initially create all the
-    Proposal steps without repeated execution of the RNG - significant speedup!
-
-    Parameters
-    ----------
-    s : :class:`numpy.ndarray`
-    """
-    def __init__(self, s):
-        self.s = np.atleast_1d(s)
-
-
-class MultivariateNormalProposal(Proposal):
-    def __call__(self, num_draws=None):
-        return np.random.multivariate_normal(
-                mean=np.zeros(self.s.shape[0]), cov=self.s, size=num_draws)
-
-
-proposal_dists = {
-    'MultivariateNormal': MultivariateNormalProposal,
-        }
+proposal_dists = {'MultivariateNormal': MultivariateNormalProposal}
 
 
 def choose_proposal(proposal_name, scale=1.):
@@ -135,7 +115,9 @@ class SMC(atext.ArrayStepSharedLLK):
                  likelihood_name='like', proposal_name='MultivariateNormal', tune=True,
                  tune_interval=100, coef_variation=1., check_bound=True, model=None,
                  random_seed=-1):
+
         warnings.warn(EXPERIMENTAL_WARNING)
+
         if random_seed != -1:
             nr.seed(random_seed)
 
@@ -147,6 +129,15 @@ class SMC(atext.ArrayStepSharedLLK):
         vars = inputvars(vars)
 
         if out_vars is None:
+            if not any(likelihood_name == RV.name for RV in model.unobserved_RVs):
+                pm._log.info(
+                    'Adding model likelihood to RVs!')
+                with model:
+                    llk = pm.Deterministic(likelihood_name, model.logpt)
+            else:
+                pm._log.info(
+                    'Using present model likelihood!')
+
             out_vars = model.unobserved_RVs
 
         out_varnames = [out_var.name for out_var in out_vars]
@@ -174,6 +165,7 @@ class SMC(atext.ArrayStepSharedLLK):
         self.accepted = 0
 
         self.beta = 0
+        self.sjs = 1
         self.stage = 0
         self.chain_index = 0
         self.resampling_indexes = np.arange(n_chains)
@@ -191,8 +183,20 @@ class SMC(atext.ArrayStepSharedLLK):
         # create initial population
         self.population = []
         self.array_population = np.zeros(n_chains)
-        for _ in range(self.n_chains):
-            self.population.append(pm.Point({v.name: v.random() for v in vars}, model=model))
+        start = model.test_point
+
+        init_rnd = {}
+        for v in vars:
+            if pm.util.is_transformed_name(v.name):
+                trans = v.distribution.transform_used.forward
+                rnd = trans(v.distribution.dist.random(size=self.n_chains, point=start))
+                init_rnd[v.name] = rnd.eval()
+            else:
+                init_rnd[v.name] = v.random(size=self.n_chains, point=start)
+
+        for i in range(self.n_chains):
+            self.population.append(pm.Point({v.name: init_rnd[v.name][i] for v in vars},
+                                            model=model))
 
         self.chain_previous_lpoint = copy.deepcopy(self.population)
 
@@ -285,23 +289,25 @@ class SMC(atext.ArrayStepSharedLLK):
             tempering parameter of the current stage
         weights : :class:`numpy.ndarray`
             Importance weights (floats)
+        sj : float
+            Mean of unnormalized weights
         """
-        low_beta = self.beta
+        low_beta = old_beta = self.beta
         up_beta = 2.
-        old_beta = self.beta
 
         while up_beta - low_beta > 1e-6:
             current_beta = (low_beta + up_beta) / 2.
-            temp = np.exp((current_beta - self.beta) * (self.likelihoods - self.likelihoods.max()))
-            cov_temp = np.std(temp) / np.mean(temp)
+            weights_un = np.exp((current_beta - self.beta) *
+                                (self.likelihoods - self.likelihoods.max()))
+            sj = np.mean(weights_un)
+            cov_temp = np.std(weights_un) / sj
             if cov_temp > self.coef_variation:
                 up_beta = current_beta
             else:
                 low_beta = current_beta
 
-        beta = current_beta
-        weights = temp / np.sum(temp)
-        return beta, old_beta, weights
+        weights = weights_un / np.sum(weights_un)
+        return current_beta, old_beta, weights, sj
 
     def calc_covariance(self):
         """Calculate trace covariance matrix based on importance weights.
@@ -333,20 +339,18 @@ class SMC(atext.ArrayStepSharedLLK):
         likelihoods : :class:`numpy.ndarray`
             Array of likelihoods of the trace end-points
         """
-        array_population = np.zeros((self.n_chains, self.ordering.dimensions))
+        array_population = np.zeros((self.n_chains, self.ordering.size))
         n_steps = len(mtrace)
 
         # collect end points of each chain and put into array
         for var, slc, shp, _ in self.ordering.vmap:
-            slc_population = mtrace.get_values(varname=var, burn=n_steps - 1, combine=True)
+            slc_population = mtrace.get_values(varname=var, burn=n_steps-1, combine=True)
             if len(shp) == 0:
                 array_population[:, slc] = np.atleast_2d(slc_population).T
             else:
                 array_population[:, slc] = slc_population
         # get likelihoods
-        likelihoods = mtrace.get_values(varname=self.likelihood_name,
-                                        burn=n_steps - 1,
-                                        combine=True)
+        likelihoods = mtrace.get_values(varname=self.likelihood_name, burn=n_steps-1, combine=True)
 
         # map end array_endpoints to dict points
         population = [self.bij.rmap(row) for row in array_population]
@@ -366,10 +370,10 @@ class SMC(atext.ArrayStepSharedLLK):
         chain_previous_lpoint : list
             all unobservedRV values, including dataset likelihoods
         """
-        array_population = np.zeros((self.n_chains, self.lordering.dimensions))
+        array_population = np.zeros((self.n_chains, self.lordering.size))
         n_steps = len(mtrace)
-        for var, (_, slc, shp, _) in zip(mtrace.varnames, self.lordering.vmap):
-            slc_population = mtrace.get_values(varname=var, burn=n_steps - 1, combine=True)
+        for _, slc, shp, _, var in self.lordering.vmap:
+            slc_population = mtrace.get_values(varname=var, burn=n_steps-1, combine=True)
             if len(shp) == 0:
                 array_population[:, slc] = np.atleast_2d(slc_population).T
             else:
@@ -419,9 +423,10 @@ class SMC(atext.ArrayStepSharedLLK):
         return outindx
 
 
-def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0, n_jobs=1,
-                 tune=None, progressbar=False, model=None, random_seed=-1, rm_flag=False):
-    """(C)ATMIP sampling algorithm (Cascading - (C) not always relevant)
+def sample_smc(n_steps, n_chains=100, step=None, start=None, homepath=None, stage=0, n_jobs=1,
+               tune_interval=10, tune=None, progressbar=False, model=None, random_seed=-1,
+               rm_flag=True):
+    """Sequential Monte Carlo sampling
 
     Samples the solution space with n_chains of Metropolis chains, where each
     chain has n_steps iterations. Once finished, the sampled traces are
@@ -440,6 +445,8 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
     ----------
     n_steps : int
         The number of samples to draw for each Markov-chain per stage
+    n_chains : int
+        Number of chains used to store samples in backend.
     step : :class:`SMC`
         SMC initialisation object
     start : List of dictionaries
@@ -448,9 +455,6 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
         Defaults to random draws from variables (defaults to empty dict)
     homepath : string
         Result_folder for storing stages, will be created if not existing.
-    chain : int
-        Chain number used to store sample in backend. If `n_jobs` is
-        greater than one, chain numbers will start here.
     stage : int
         Stage where to start or continue the calculation. It is possible to
         continue after completed stages (stage should be the number of the
@@ -460,6 +464,8 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
         internal parallelisation. Sometimes this is more efficient especially
         for simple models.
         step.n_chains / n_jobs has to be an integer number!
+    tune_interval : int
+        Number of steps to tune for
     tune : int
         Number of iterations to tune, if applicable (defaults to None)
     progressbar : bool
@@ -485,12 +491,14 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
     warnings.warn(EXPERIMENTAL_WARNING)
 
     model = modelcontext(model)
-    step.n_steps = int(n_steps)
+
     if random_seed != -1:
         nr.seed(random_seed)
 
     if step is None:
-        raise TypeError('Argument `step` has to be a SMC step object.')
+        pm._log.info('Argument `step` is None. Auto-initialising step object '
+                     'using given/default parameters.')
+        step = SMC(n_chains=n_chains, tune_interval=tune_interval, model=model)
 
     if homepath is None:
         raise TypeError('Argument `homepath` should be path to result_directory.')
@@ -507,8 +515,10 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
             step.population = start
 
     if not any(step.likelihood_name in var.name for var in model.deterministics):
-        raise TypeError('Model (deterministic) variables need to contain a variable %s '
-                        'as defined in `step`.' % step.likelihood_name)
+        raise TypeError('Model (deterministic) variables need to contain a variable {} as defined '
+                        'in `step`.'.format(step.likelihood_name))
+
+    step.n_steps = int(n_steps)
 
     stage_handler = atext.TextStage(homepath)
 
@@ -524,25 +534,8 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
         draws = step.n_steps
 
     stage_handler.clean_directory(stage, None, rm_flag)
-    with model:
-        chains = stage_handler.recover_existing_results(stage, draws, step, n_jobs)
-        if chains is not None:
-            rest = len(chains) % n_jobs
-            if rest > 0:
-                pm._log.info('Fixing %i chains ...' % rest)
-                chains, rest_chains = chains[:-rest], chains[-rest:]
-                # process traces that are not a multiple of n_jobs
-                sample_args = {
-                    'draws': draws,
-                    'step': step,
-                    'stage_path': stage_handler.stage_path(stage),
-                    'progressbar': progressbar,
-                    'model': model,
-                    'n_jobs': rest,
-                    'chains': rest_chains}
 
-                _iter_parallel_chains(**sample_args)
-                pm._log.info('Back to normal!')
+    chains = stage_handler.recover_existing_results(stage, draws, step)
 
     with model:
         while step.beta < 1:
@@ -556,23 +549,22 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
             pm._log.info('Beta: %f Stage: %i' % (step.beta, step.stage))
 
             # Metropolis sampling intermediate stages
-            chains = stage_handler.clean_directory(stage, chains, rm_flag)
-            sample_args = {
-                    'draws': draws,
-                    'step': step,
-                    'stage_path': stage_handler.stage_path(step.stage),
-                    'progressbar': progressbar,
-                    'model': model,
-                    'n_jobs': n_jobs,
-                    'chains': chains}
+            chains = stage_handler.clean_directory(step.stage, chains, rm_flag)
+            sample_args = {'draws': draws,
+                           'step': step,
+                           'stage_path': stage_handler.stage_path(step.stage),
+                           'progressbar': progressbar,
+                           'model': model,
+                           'n_jobs': n_jobs,
+                           'chains': chains}
 
             _iter_parallel_chains(**sample_args)
 
             mtrace = stage_handler.load_multitrace(step.stage)
 
-            step.population, step.array_population, step.likelihoods = step.select_end_points(
-                mtrace)
-            step.beta, step.old_beta, step.weights = step.calc_beta()
+            step.population, step.array_population, step.likelihoods = step.select_end_points(mtrace)
+            step.beta, step.old_beta, step.weights, sj = step.calc_beta()
+            step.sjs *= sj
 
             if step.beta > 1.:
                 pm._log.info('Beta > 1.: %f' % step.beta)
@@ -596,24 +588,31 @@ def ATMIP_sample(n_steps, step=None, start=None, homepath=None, chain=0, stage=0
         # Metropolis sampling final stage
         pm._log.info('Sample final stage')
         step.stage = -1
-        temp = np.exp((1 - step.old_beta) * (step.likelihoods - step.likelihoods.max()))
-        step.weights = temp / np.sum(temp)
+        chains = stage_handler.clean_directory(step.stage, chains, rm_flag)
+        weights_un = np.exp((1 - step.old_beta) * (step.likelihoods - step.likelihoods.max()))
+        step.weights = weights_un / np.sum(weights_un)
         step.covariance = step.calc_covariance()
         step.proposal_dist = choose_proposal(step.proposal_name, scale=step.covariance)
         step.resampling_indexes = step.resample()
         step.chain_previous_lpoint = step.get_chain_previous_lpoint(mtrace)
 
+        sample_args['draws'] = n_steps
         sample_args['step'] = step
         sample_args['stage_path'] = stage_handler.stage_path(step.stage)
         sample_args['chains'] = chains
         _iter_parallel_chains(**sample_args)
 
         stage_handler.dump_atmip_params(step)
-        return stage_handler.load_multitrace(step.stage, model=model)
+
+        model.marginal_likelihood = step.sjs
+        return stage_handler.create_result_trace(step.stage,
+                                                 idxs=range(n_steps),
+                                                 step=step,
+                                                 model=model)
 
 
-def _sample(draws, step=None, start=None, trace=None, chain=0, tune=None,
-            progressbar=True, model=None, random_seed=-1):
+def _sample(draws, step=None, start=None, trace=None, chain=0, tune=None, progressbar=True,
+            model=None, random_seed=-1):
 
     sampling = _iter_sample(draws, step, start, trace, chain,
                             tune, model, random_seed)
